@@ -31,8 +31,9 @@ func getGitCommitCmd(cmd *cobra.Command) []string {
 	commitCmd := []string{}
 	noEdit, _ := cmd.Flags().GetBool("no-edit")
 	message, _ := cmd.Flags().GetStringArray("message")
+	flags := cmd.Flags()
 	for _, name := range boolFlags {
-		flag, _ := cmd.Flags().GetBool(name)
+		flag, _ := flags.GetBool(name)
 		if flag {
 			commitCmd = append(commitCmd, "--"+name)
 		}
@@ -77,13 +78,25 @@ func doCommit(message string, dryRun bool, commitParams []string) {
 	}
 }
 
+// 0000 0001 : invalid type
+// 0000 0010 : missing type
+// 0000 0100 : invalid scope
+// 0000 1000 : missing description
+type ValidationErrors = uint8
+
+const (
+	InvalidType        uint8 = 1 << 0
+	MissingType        uint8 = 1 << 1
+	InvalidScope       uint8 = 1 << 2
+	MissingDescription uint8 = 1 << 3
+)
+
 // run the conventional-commit helper logic. This may/not break into the TUI.
-func mainMode(cmd *cobra.Command, args []string) {
-	cfg := config.Lookup(config.Init())
+func mainMode(cmd *cobra.Command, args []string, cfg *config.Cfg) {
+
 	commitParams := getGitCommitCmd(cmd)
-	dryRun, _ := cmd.Flags().GetBool("dry-run")
 	committingAllChanges, _ := cmd.Flags().GetBool("all")
-	if !dryRun && !committingAllChanges {
+	if !cfg.DryRun && !committingAllChanges {
 		buf := &bytes.Buffer{}
 		process := exec.Command("git", "diff", "--name-only", "--cached")
 		process.Stdout = buf
@@ -105,13 +118,27 @@ func mainMode(cmd *cobra.Command, args []string) {
 	} else {
 		cc, _ = parser.ParseAsMuchOfCCAsPossible((strings.Join(args, " ")))
 	}
-	valid := cc.MinimallyValid() &&
-		cc.ValidCommitType(cfg.CommitTypes) &&
-		(cc.ValidScope(cfg.Scopes) || cc.Scope == "")
-	if !valid {
+	var validationErrors ValidationErrors = 0
+	if cc.Type == "" {
+		validationErrors |= InvalidType
+	} else {
+		if _, valid := cfg.CommitTypes.Get(cc.Type); !valid {
+			validationErrors |= InvalidType
+		}
+	}
+	if cc.Scope != "" {
+		if _, valid := cfg.Scopes.Get(cc.Scope); !valid {
+			validationErrors |= InvalidScope
+		}
+	}
+	if cc.Description == "" {
+		validationErrors |= MissingDescription
+	}
+
+	if validationErrors != 0 {
 		choice := make(chan string, 1)
 		m := initialModel(choice, cc, cfg)
-		ui := tea.NewProgram(m, tea.WithInputTTY())
+		ui := tea.NewProgram(m, tea.WithInputTTY(), tea.WithAltScreen())
 		if err := ui.Start(); err != nil {
 			log.Fatal(err)
 		}
@@ -128,60 +155,97 @@ func mainMode(cmd *cobra.Command, args []string) {
 			if err != nil {
 				log.Fatalf("unable to write to file %s: %+v", f, err)
 			}
-			doCommit(result, dryRun, commitParams)
+			doCommit(result, cfg.DryRun, commitParams)
 		}
 	} else {
-		doCommit(cc.ToString(), dryRun, commitParams)
+		doCommit(cc.ToString(), cfg.DryRun, commitParams)
 	}
 }
 
+func redoMessage(cmd *cobra.Command) {
+	flags := cmd.Flags()
+	msg, _ := flags.GetStringArray("message")
+	if len(msg) > 0 {
+		log.Fatal("-m|--message is incompatible with --redo")
+	}
+	commitMessagePath := config.GetCommitMessageFile()
+	data, err := os.ReadFile(commitMessagePath)
+	// TODO: ignore commented lines in git-commit file
+	if err != nil {
+		fmt.Printf("file not found: %s", commitMessagePath)
+		os.Exit(127)
+	}
+	empty := true
+	for _, line := range strings.Split(
+		strings.Trim(string(data), "\r\n\t "),
+		"\n",
+	) {
+		if !strings.HasPrefix(strings.TrimLeft(line, " \t\r\n"), "#") {
+			empty = false
+			break // ok
+		}
+	}
+	if empty {
+		log.Fatalf("Empty commit message: %s", commitMessagePath)
+	}
+
+	flags.Set("message", string(data))
+}
+
 var Cmd = &cobra.Command{
 	Use:   "git-cc",
 	Short: "write conventional commits",
 	// not using cobra subcommands since they prevent passing arbitrary arguments
 	Run: func(cmd *cobra.Command, args []string) {
-		version, _ := cmd.Flags().GetBool("version")
-		if version {
+		flags := cmd.Flags()
+		if version, _ := flags.GetBool("version"); version {
 			versionMode()
 			os.Exit(0)
-		}
-		genCompletion, _ := cmd.Flags().GetBool("generate-shell-completion")
-		if genCompletion {
+		} else if genCompletion, _ := flags.GetBool("generate-shell-completion"); genCompletion {
 			generateShellCompletion(cmd, args)
 			os.Exit(0)
-		}
-		genManPage, _ := cmd.Flags().GetBool("generate-man-page")
-		if genManPage {
+		} else if genManPage, _ := flags.GetBool("generate-man-page"); genManPage {
 			generateManPage(cmd, args)
 			os.Exit(0)
+		} else {
+			dryRun, _ := cmd.Flags().GetBool("dry-run")
+			cfg, err := config.Init(dryRun)
+			if err != nil {
+				log.Fatal(err)
+			}
+			if redo, _ := flags.GetBool("redo"); redo {
+				redoMessage(cmd)
+			}
+			mainMode(cmd, args, cfg)
 		}
-		mainMode(cmd, args)
 	},
 }
 
 func init() {
-	Cmd.Flags().BoolP("help", "h", false, "print the usage of git-cc")
-	Cmd.Flags().Bool("dry-run", false, "Only print the resulting conventional commit message; don't commit.")
-	Cmd.Flags().StringArrayP("message", "m", []string{}, "pass a complete conventional commit. If valid, it'll be committed without editing.")
-	Cmd.Flags().Bool("version", false, "print the version")
+	flags := Cmd.Flags()
+	flags.BoolP("help", "h", false, "print the usage of git-cc")
+	flags.Bool("dry-run", false, "Only print the resulting conventional commit message; don't commit.")
+	flags.Bool("redo", false, "Reuse your last commit message")
+	flags.StringArrayP("message", "m", []string{}, "pass a complete conventional commit. If valid, it'll be committed without editing.")
+	flags.Bool("version", false, "print the version")
 	// TODO: accept more of git commit's flags; see https://git-scm.com/docs/git-commit
 	// likely: --cleanup=<mode>
 	// more difficult, and possibly better done manually: --amend, -C <commit>
 	// --reuse-message=<commit>, -c <commit>, --reedit-message=<commit>,
 	// --fixup=<commit>, --squash=<commit>
-	Cmd.Flags().String("author", "", "delegated to git-commit")
-	Cmd.Flags().String("date", "", "delegated to git-commit")
-	Cmd.Flags().BoolP("all", "a", false, "see the git-commit docs for --all|-a")
-	Cmd.Flags().BoolP("signoff", "s", false, "see the git-commit docs for --signoff|-s")
-	Cmd.Flags().Bool("no-gpg-sign", false, "see the git-commit docs for --no-gpg-sign")
-	Cmd.Flags().Bool("no-post-rewrite", false, "Bypass the post-rewrite hook")
-	Cmd.Flags().Bool("no-edit", false, "Use the selected commit message without launching an editor.")
-	Cmd.Flags().BoolP("no-verify", "n", false, "Bypass git hooks")
-	Cmd.Flags().Bool("verify", true, "Ensure git hooks run")
+	flags.String("author", "", "delegated to git-commit")
+	flags.String("date", "", "delegated to git-commit")
+	flags.BoolP("all", "a", false, "see the git-commit docs for --all|-a")
+	flags.BoolP("signoff", "s", false, "see the git-commit docs for --signoff|-s")
+	flags.Bool("no-gpg-sign", false, "see the git-commit docs for --no-gpg-sign")
+	flags.Bool("no-post-rewrite", false, "Bypass the post-rewrite hook")
+	flags.Bool("no-edit", false, "Use the selected commit message without launching an editor.")
+	flags.BoolP("no-verify", "n", false, "Bypass git hooks")
+	flags.Bool("verify", true, "Ensure git hooks run")
 	// https://git-scm.com/docs/git-commit#Documentation/git-commit.txt---no-verify
-	Cmd.Flags().Bool("no-signoff", true, "Don't add a a `Signed-off-by` trailer to the commit message")
-	Cmd.Flags().Bool("generate-man-page", false, "Generate a man page in your manpath")
-	Cmd.Flags().Bool(
+	flags.Bool("no-signoff", true, "Don't add a a `Signed-off-by` trailer to the commit message")
+	flags.Bool("generate-man-page", false, "Generate a man page in your manpath")
+	flags.Bool(
 		"generate-shell-completion",
 		false,
 		"print a bash/zsh/fish/powershell completion script to stdout",
