@@ -1,6 +1,7 @@
 package single_select
 
 import (
+	"fmt"
 	"strings"
 
 	"github.com/charmbracelet/bubbles/textinput"
@@ -12,29 +13,29 @@ import (
 )
 
 type Model struct {
-	Options         []string
-	Hints           []string
-	match           func(*Model, string, string) bool
-	filtered        [][2]string
-	matched         [][2]string
+	Options  []string
+	Hints    []string
+	match    func(*Model, string, string) bool
+	filtered [][2]string
+	matched  [][2]string
+	// the offset (0-indexed) of the selection within the options
 	Cursor          int
 	maxOptionLength int
 	context         string
-	Width           int // in runes
-	Height          int // in lines
-	textInput       textinput.Model
+	// in runes
+	Width int
+	// in lines
+	Height    int
+	textInput textinput.Model
 }
 
 func (m Model) Init() tea.Cmd {
 	return nil
 }
 
-func NewModel(context string, value string, options []map[string]string, match func(*Model, string, string) bool) Model {
-	values, hints := []string{}, []string{}
-	for _, option := range options {
-		for value, hint := range option {
-			values, hints = append(values, value), append(hints, hint)
-		}
+func NewModel(context string, value string, options []string, hints []string, match func(*Model, string, string) bool) Model {
+	if len(options) != len(hints) {
+		panic(fmt.Errorf("len(hints) %d != %d len(options)", len(hints), len(options)))
 	}
 	input := textinput.NewModel()
 	input.Placeholder = "type to select"
@@ -45,7 +46,7 @@ func NewModel(context string, value string, options []map[string]string, match f
 
 	result := Model{
 		context:   context,
-		Options:   values,
+		Options:   options,
 		Hints:     hints,
 		textInput: input,
 		match:     match,
@@ -87,7 +88,7 @@ func (m Model) maxOptLen() int {
 		}
 	}
 	m.maxOptionLength = max
-	return max
+	return m.maxOptionLength
 }
 
 func MatchStart(m *Model, query string, option string) bool {
@@ -147,6 +148,28 @@ func Update(msg tea.Msg, model Model) (Model, tea.Cmd) {
 			model.Cursor = 0
 			return model, cmd
 		}
+	case tea.MouseEvent:
+		switch msg.Type {
+		case tea.MouseWheelUp:
+			if model.Cursor > 0 {
+				model.Cursor -= 1
+			} else {
+				model.Cursor = len(model.matched) - 1
+			}
+			return model, cmd
+		case tea.MouseWheelDown:
+			if model.Cursor < len(model.matched)-1 {
+				model.Cursor += 1
+			} else {
+				model.Cursor = 0
+			}
+			return model, cmd
+		default:
+			model.textInput, cmd = model.textInput.Update(msg)
+			model.matched, model.filtered = model.filter(model.textInput.Value())
+			model.Cursor = 0
+			return model, cmd
+		}
 	case tea.WindowSizeMsg:
 		model.Height = msg.Height
 		model.Width = msg.Width
@@ -155,7 +178,6 @@ func Update(msg tea.Msg, model Model) (Model, tea.Cmd) {
 	default:
 		model.textInput, cmd = model.textInput.Update(msg)
 		model.matched, model.filtered = model.filter(model.textInput.Value())
-		model.Cursor = 0
 		return model, cmd
 	}
 }
